"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("./events");
class AggregateRootImpl {
    constructor(state, version) {
        this.__currentPromise = Promise.resolve();
        this.__error = null;
        this.__state = null;
        this.__markedForDeletion = false;
        this.__events = [];
        this.__previousVersion = 0;
        if (state instanceof Error) {
            this.__error = state;
        }
        else if (state) {
            if (!version || version < 1) {
                this.__error = new Error('Version was missing when initializing state');
                return;
            }
            this.__previousVersion = version;
            this.__state = state;
        }
    }
    get state() {
        return this.__currentPromise.then(() => this.__state);
    }
    get error() {
        return this.__currentPromise.then(() => this.__error);
    }
    get version() {
        return this.__previousVersion;
    }
    get events() {
        return this.__currentPromise.then(() => this.__events);
    }
    mutation(mutator) {
        return (cmd) => {
            this.__currentPromise = this.__currentPromise
                .then(async () => {
                if (!this.__error) {
                    try {
                        this.__state = await mutator(this.__state, cmd, this);
                    }
                    catch (e) {
                        this.__error = e;
                    }
                }
            });
            return this;
        };
    }
    factory(factoryFunc) {
        return async (cmd) => {
            if (this.__error) {
                return new AggregateRootImpl(this.__error);
            }
            try {
                return factoryFunc(this.__state, cmd);
            }
            catch (e) {
                return new AggregateRootImpl(e);
            }
        };
    }
    'delete'() {
        this.__currentPromise = this.__currentPromise
            .then(() => {
            let types = [];
            if (this.__state) {
                if (Array.isArray(this.__state['@type'])) {
                    types = this.__state['@type'];
                }
                else {
                    types = [this.__state['@type']];
                }
            }
            this.emit('AggregateDeleted', {
                types,
            });
            this.__markedForDeletion = true;
        });
        return this;
    }
    commit(repo) {
        return this.__currentPromise.then(() => {
            if (this.__error) {
                throw this.__error;
            }
            if (!this.__state) {
                throw new Error('Cannot commit null state.');
            }
            return repo.save(this, this.__previousVersion + 1)
                .then(() => {
                if (this.__markedForDeletion) {
                    return repo.delete(this.__state['@id']);
                }
            })
                .then(() => {
                this.__events.forEach(e => events_1.emit(e.name, {
                    id: this.__state['@id'],
                    ...e,
                }));
                return this.__state;
            });
        });
    }
    emit(name, data) {
        this.__events.push({
            name,
            data,
        });
    }
}
exports.AggregateRootImpl = AggregateRootImpl;
